/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.kernel.axiom.atomic;

import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.transport.Communication;
import com.vii.brillien.kernel.axiom.transport.Mediator;

import java.util.List;
import java.util.Map;

/**
 * Control board for Presences to orchestrate a given Presence by versions
 */
public interface PresenceBoard<P extends PresenceManager, C extends Communication> extends Unit<P, C, Object> {

    void                            activatePresenceBoard() throws BrillienException;

    String                          getDefaultVersion() throws BrillienException;

    List<String>                    getVersions() throws BrillienException;

    Map<String, P>                  getPresenceManagers() throws BrillienException;

    P                               getDefaultPresenceManager( ) throws BrillienException;

    P                               getPresenceManager( String apiVersion ) throws BrillienException;

    /**
     * Publishes the given Presence Manager associated with the given version
     */
    void                            publishPresenceManager( String presenceName, P manager, Class presenceClass, String apiVersion ) throws BrillienException;

    /**
     * Deactivates the PresenceManager identified with the given version
     */
    void                            revokePresenceManager( String apiVersion ) throws BrillienException;

    /**
     * Retrieves a mediator representing the transport layer
     */
    <T> Mediator<C, T>              getMediator();

    /**
     * Presences let the PresenceManager to know about the messages sent through this method
     */
    void                            markMessageToSend(C shared, C message);

}